import asyncio
import threading
import requests
from bs4 import BeautifulSoup

try:
    from utils import time_detect
    from wraps import repeat_every_30_seconds
except ModuleNotFoundError:
    from nicoinfo.plugins.nicoinfo.utils import time_detect
    from nicoinfo.plugins.nicoinfo.wraps import repeat_every_30_seconds


class auth_auth_info:
    def __init__(self, uid: str):
        self.uid = uid
        self.api_url = f"https://www.nicovideo.jp/user/{uid}/video?rss=2.0"
        self.items, self.auth = self._get_videos_info()

    def _get_videos_info(self):
        response = requests.get(self.api_url)
        response.raise_for_status()

        soup = BeautifulSoup(response.content, "lxml")
        item = soup.find_all("item")
        auth = soup.select('channel dc\\:creator')
        return item, auth

    def get_last_video(self):
        item = self.items[0]
        title = item.title.text
        video_url = item.link.next_sibling.strip()
        real_url = video_url.split('?')[0]
        p_nico_description = item.find("p", class_="nico-description").text

        p_nico_info = item.find("p", class_="nico-info")
        video_id = real_url.split('/')[-1]
        # 如果找到了<p>标签，则提取日期信息
        if p_nico_info:
            date_info = p_nico_info.find("strong", class_="nico-info-date").text
        else:
            date_info = "无日期信息"

        last_video = {
            'title': title,
            'date_info': date_info,
            'url': real_url,
            'description': p_nico_description,
            'auth': self.auth[0].text,
            'sm': video_id
        }
        return last_video

    def get_videos_list(self):
        videos_list = {}
        for item in self.items:
            title = item.title.text
            video_url = item.link.next_sibling.strip()
            real_url = video_url.split('?')[0]
            p_nico_description = item.find("p", class_="nico-description").text

            p_nico_info = item.find("p", class_="nico-info")
            video_id = real_url.split('/')[-1]
            # 如果找到了<p>标签，则提取日期信息
            if p_nico_info:
                date_info = p_nico_info.find("strong", class_="nico-info-date").text
            else:
                date_info = "无日期信息"
            video = {
                'title': title,
                'date_info': date_info,
                'url': real_url,
                'description': p_nico_description,
                'auth': self.auth[0].text,
                'sm': video_id
            }
            videos_list[video_id] = video
        videos_list['auth'] = self.auth[0].text
        return videos_list


async def detect_new_video(uid):
    video = auth_auth_info(uid)
    last_video = video.get_last_video()
    if await time_detect(last_video['date_info']):
        return last_video


class author_container:
    def __init__(self):
        self.auth_list = {}

    def sub_auth(self, uid: str, need_last: bool = False):
        _ = auth_auth_info(uid)
        self.auth_list[uid] = _.get_videos_list()
        if need_last:
            return _.get_last_video()

    async def get_new_video(self):
        for auth in self.auth_list:
            await detect_new_video(auth)

    def update_list(self, data: dict):
        self.auth_list.update(data)


class subscriber:
    def __init__(self, member: str, is_private: bool = False):
        self.member = member
        self.is_private = is_private
        self.container = author_container()
        self.bot = None

    def get_the_bot(self, bot):
        self.bot = bot

    def out_of_bot(self):
        self.bot = None

    def sub(self, uid, need_last: bool = False):
        self.container.sub_auth(uid, need_last)

    def desub(self, uid):
        del self.container.auth_list[uid]

    def if_suber(self, uid):
        if uid in self.container.auth_list:
            return True
        else:
            return False

    def get_sub_list(self):
        return self.container.auth_list

    async def update_sub_list(self, suber):
        last_video = await detect_new_video(suber)
        if not last_video:
            print(suber + ":无视频更新。" + "会话：" + self.member)
            return None
        if last_video['sm'] not in self.container.auth_list[suber]:
            self.container.auth_list[suber]['sm'] = last_video
            if await time_detect(last_video['date_info']):
                print(last_video)
                return last_video

    def get_json_data(self) -> dict:
        subscriber_json = {
            "member": self.member,
            "is_private": self.is_private,
            "container": self.container.auth_list
        }
        return subscriber_json

    @staticmethod
    def create_from_json_data(json_data: dict):
        subscriber_instance = subscriber(json_data["member"], json_data["is_private"])
        subscriber_instance.container.update_list(json_data["container"])
        return subscriber_instance

    @repeat_every_30_seconds
    async def continuously_update(self, ):
        results = await asyncio.gather(*(self.update_sub_list(suber) for suber in self.container.auth_list))
        if self.bot:
            for result in results:
                if result and type(result) is dict:
                    from nicoinfo.plugins.nicoinfo.usage import send_last_video_to_private_or_group as send_message
                    await send_message(self.bot, self.member, result, self.is_private)


async def subscribers_run(subscribers: dict[str, subscriber]):
    tasks = [subscriber_instance.continuously_update() for subscriber_instance in subscribers.values()]
    await asyncio.gather(*tasks)


def start_asyncio_loop(loop, coroutine):
    asyncio.set_event_loop(loop)
    loop.run_until_complete(coroutine)


async def main():
    subscriber_instance = subscriber('test')
    subscriber_instance.sub('4970449')
    subscriber_instance.sub('51546188')
    subscriber_instance2 = subscriber('test2')
    subscriber_instance2.sub('125247562')
    subscribers = {"114514": subscriber_instance,
                   "1919810": subscriber_instance2}
    # await subscriber_instance.continuously_update()
    await subscribers_run(subscribers)


if __name__ == '__main__':
    print(111)  # 主程序可以继续执行其他操作
    new_loop = asyncio.new_event_loop()
    threading.Thread(target=start_asyncio_loop, args=(new_loop, main())).start()
    print(222)



import requests
from nonebot.params import CommandArg
from nonebot import on_command
from nonebot.adapters.onebot.v11 import Bot, Event, Message
from .nico_auth import auth_auth_info, subscriber
from .usage import load_subscriptions, save_subscriptions, send_image_and_text, get_samune_from_video, \
    send_image_from_url
from .usage import logger, video_info_to_str
import os

from .utils import get_chat_id

subscriptions_filename = "subscriptions.json"
if os.path.exists(subscriptions_filename):
    subscriptions = load_subscriptions(subscriptions_filename)
else:
    subscriptions = {}


async def get_subscriber(event: Event) -> subscriber:
    logger.info("用户ID:" + str(event.get_user_id()))
    logger.info("is_tome:" + str(event.is_tome()))
    chat_id = get_chat_id(event)
    logger.info("chat_id:" + chat_id)
    if chat_id not in subscriptions:
        # 如果字典中没有该群聊或私人会话的订阅器实例，创建一个并添加到字典中
        is_private = event.is_tome()
        subscriptions[chat_id] = subscriber(chat_id, is_private)

    return subscriptions[chat_id]


sub_cmd = on_command("sub", aliases={"订阅"}, block=True)


@sub_cmd.handle()
async def sub_command(bot: Bot, event: Event):
    args = str(event.get_message()).split(' ')[1]
    try:
        uid = int(args)
    except Exception:
        await bot.send(event, Message("参数错误"))
        return
    subscriber_instance = await get_subscriber(event)
    subscriber_instance.sub(uid)
    await bot.send(event, Message("订阅成功"))
    logger.info(subscriptions)
    save_subscriptions(subscriptions, subscriptions_filename)
    logger.info("参数：" + args)
    if "last" or "need last" in args:
        a = auth_auth_info(str(uid))
        last = a.get_last_video()
        await auth_last_video.finish(video_info_to_str(last))


desub_cmd = on_command("desub", aliases={"取消订阅", "cancel subscribe", "del"}, block=True, priority=5)
@desub_cmd.handle()
async def desub_command(bot: Bot, event: Event):
    del_list = []
    chat_id = get_chat_id(event)
    args = str(event.get_message()).split(' ')
    logger.debug(args)
    for arg in args:
        if arg.isdigit():
            if subscriptions[chat_id].if_suber(arg):
                subscriptions[chat_id].desub(arg)
                del_list.append(arg)
    if del_list:
        del_info = ",".join(del_list)
        await bot.send(event, "已删除订阅" + del_info)
        save_subscriptions(subscriptions, subscriptions_filename)
    else:
        await bot.send(event, "订阅号错误")


# 其他命令和方法也可以使用 get_subscriber 函数来获取相应的订阅器实例
get_list = on_command("list", aliases={"sub_list", "订阅列表"}, block=True)


@get_list.handle()
async def get_list(bot: Bot, event: Event):
    args = str(event.get_message()).strip()
    message = ""
    subscriber_instance = await get_subscriber(event)
    subscribers_list = subscriber_instance.get_sub_list()
    for suber in subscribers_list:
        message += (str(suber) + ':' + subscribers_list[suber]['auth'] + '\n')

    await bot.send(event, Message("当前订阅列表：\n" + message))


auth_last_video = on_command("last", aliases={"最后投稿", "last video", "最新"}, priority=5)


@auth_last_video.handle()
async def last_video(bot: Bot, event: Event, args: Message = CommandArg()):
    if not args:
        await auth_last_video.finish("需要作者uid")
    args = str(args).split(' ')
    fast_run = ("fast" or "快速" or "无封面" or "fast run") in args
    logger.info('快速启动:' + str(fast_run))
    for arg in args:
        logger.info(arg)
        if str(arg).isdigit():
            a = auth_auth_info(arg)
            try:
                last = a.get_last_video()
            except requests.exceptions.ConnectTimeout:
                await bot.send(event, Message("链接失败，请稍后重试"))
                break
            text = video_info_to_str(last)
            if fast_run:
                logger.info(text)
                await bot.send(event, Message(text))
                continue
            video = await get_samune_from_video(last)
            logger.debug("封面信息" + str(video))
            await send_image_and_text(bot, event, text, str(video))
            os.remove(video)


send_img = on_command("img", aliases={'image'})


@send_img.handle()
async def send_img_(bot: Bot, event: Event, args: Message = CommandArg()):
    for arg in args:
        await send_image_from_url(bot, event, str(arg))





import asyncio
import os
import random

# from aiohttp import ClientSession
from nonebot.adapters.onebot.v11 import Bot, Event
import aiohttp
import requests
from bs4 import BeautifulSoup
from datetime import datetime
from lxml import etree

try:
    from nicoinfo.plugins.nicoinfo.usage import send_image_from_ab_path, is_full_color
    from nicoinfo.plugins.nicoinfo.wraps import retry_async
except ValueError:
    from wraps import retry_async

header = {
    'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36',
    'Cookie': 'nicosid=1633700428.452220814; _ts_yjad=1633700428228; _ss_pp_id=9b353ec49aa360431471646182137066; _cc_id=f58d08fa37a8aaae434b30e83797ecfd; lang=ja-jp; area=JP; _ga_8W314HNSE8=GS1.1.1653933007.94.1.1653934013.0; _gcl_au=1.1.553504617.1659349543; pbjs_sharedId=59bf20ff-a27d-4f03-804f-b7920a216078; _fbp=fb.1.1665462893682.174282674; __gads=ID=2d7a6b88f86b06c7:T=1633700444:S=ALNI_MZV9KgdWjSDI3uzQgPo9m7Ep8C-UA; _td=ceef4192-d12e-43ba-b7c5-ed24cc177792; common-header-oshirase-open-date=2022-10-25T13:04:19.965Z; __gpi=UID=00000450e6f57b88:T=1649410308:RT=1666703067:S=ALNI_MYaaH1zOxhAP4tI_sasBqNVtWHvpQ; nico_gc=tg_s%3Df%26tg_o%3Dd; cto_bundle=0-T18V94MmR6am1ZWmVJOWt4YmY1NE5MYW5FTEtsOVRrJTJGMEklMkIwY3lPbnlSZnJsbjlWTWFDdWFweEdLeFlDT25BWXo1U29nbVNWTHRSamZKOXNRdjElMkJDcUVTZE1uVU1PVmxzSFhZT1RUTW9vcHo0UHUzdFR6YlBjQ29rNFA3Vm9BZDBCVmI5c3IwZXNIZUFLdWttVEF3MXVReXclM0QlM0Q; optimizelyEndUserId=oeu1666928747475r0.3711834985684135; _gid=GA1.2.1402785906.1667051185; _ga_7W6WKHGQTW=GS1.1.1667051183.4.1.1667051767.58.0.0; _ga_G7379W9VJ0=GS1.1.1667051183.4.1.1667051767.60.0.0; _gat_UA-88451119-5=1; _gat_NicoGoogleTagManager=1; mfa_session=87722391_m8gVOCWbbtAtvvswitvaVU2KIKiJueXI; _ga=GA1.1.747558350.1633700427; user_session=user_session_87722391_f3b01de072c501c84b48f8005b2a74be53bde8445d11f1469bfd8f6bf548eb62; user_session_secure=ODc3MjIzOTE6TTVKNll2ajhCZC5TVTVROU1UaHhKMnFRNXV6dWFWeW9vZXZ4SUUxNDVaVA; _ga_5LM4HED1NJ=GS1.1.1667051185.267.1.1667051825.1.0.0'
}


@retry_async(retries=3, delay=1)
async def get_im_date_info(id):
    url = f'http://seiga.nicovideo.jp/api/illust/info?id={id}'
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            response_text = await response.text()
            soup = BeautifulSoup(response_text, 'lxml')
            date = soup.find("created").text
    return date


def to_year_and_month(date):
    date_object = datetime.strptime(date, "%Y-%m-%d %H:%M:%S")

    # 从datetime对象获取年份和月份
    year = date_object.year
    month = date_object.month

    return year, month


def get_point_of_cookie(count_list: list):
    point = 1 * int(count_list[0]) + 2 * int(count_list[1]) + 200 * int(count_list[2])
    return point


def get_cookie_elements(tag: str, sort: str = "image_view", least_point=20000):
    page = random.randint(1, 200)
    url = f"https://seiga.nicovideo.jp/tag/{tag}?sort={sort}&page={page}"
    headers = {'User-Agent': 'Mozilla/5.0'}
    response = requests.get(url, headers=headers)
    if response.status_code == 200:
        root = etree.HTML(response.content)
        # 使用 XPath 选择元素
        im_list = []
        elements = root.xpath('//li[@class="list_item list_no_trim2"]/a')
        for element in elements:
            lllust = element.xpath('.//ul[@class="illust_count"]/li/text()')
            point = get_point_of_cookie(lllust)
            im_hao = element.attrib['href'].split('/')[-1]
            if point >= least_point:
                im_list.append(im_hao)
        if im_list == []:
            return False
        return im_list


async def download_image(url, local_path):
    # async with aiohttp.ClientSession() as session:
    #     async with session.get(url) as response:
    #         if response.status == 200:
    #             directory = os.path.dirname(local_path)
    #             if not os.path.exists(directory):
    #                 os.makedirs(directory)
    #             with open(local_path, 'wb') as file:
    #                 file.write(await response.read())
    #         else:
    #             return False
    response = requests.get(url, headers=header)
    if response.status_code == 200:
        directory = os.path.dirname(local_path)
        if not os.path.exists(directory):
            os.makedirs(directory)
        with open(local_path, 'wb') as file:
            file.write(response.content)
    else:
        return False


def get_image_url(base_url, file_id, possible_extensions):
    for ext in possible_extensions:
        url = f"{base_url}{file_id}.{ext}"
        response = requests.get(url)
        if response.status_code == 200 and 'image' in response.headers['Content-Type']:
            return url
    return None


async def get_url_im_download(im):
    id = im.split("im")[1]
    url = f"https://seiga.nicovideo.jp/image/source/{id}"
    # async with ClientSession() as session:
    #     async with session.get(url, headers=header) as response:
    #         text = await response.text()

    text = requests.get(url, headers=header).text
    soup = BeautifulSoup(text, "html.parser")
    div = soup.find("div", {"class": "illust_view_big"})
    data_src = div.get("data-src")
    return data_src


async def download_with_im(im, local_path="image"):
    print("下载", im)
    data_src = await get_url_im_download(im)
    # 使用 os.path.join 来创建路径
    path = os.path.abspath(os.path.join(local_path, im))
    await download_image(data_src, path)

async def generate_cookie_image_url(im: str):
    id = im.split('im')[1]
    date = await get_im_date_info(id)
    year, month = to_year_and_month(date)
    url = f"https://cdn.jsdelivr.net/gh/DirtyCookies/CookieImages@main/{year}/{month}/{im}"
    return url


async def download_with_im_(im, local_path="image"):
    print("下载", im)
    url = await generate_cookie_image_url(im)
    png = ['png', 'jpeg', 'gif']

    # 这里使用 os.path.join 方法来构建路径
    tasks = [asyncio.create_task(
        download_image(url + "." + ext, os.path.abspath(os.path.join(local_path, f'{im}.{ext}'))))
        for ext in png]
    completed = False
    while not completed:
        done, pending = await asyncio.wait(tasks, return_when=asyncio.FIRST_COMPLETED)
        for task in done:
            result = task.result()
            if result is not False:
                completed = True
                for pending_task in pending:
                    pending_task.cancel()
                return
        tasks = pending


async def download_muti_im(im_list: list, local="image"):
    print("多下载")
    tasks = [asyncio.create_task(
        download_with_im(im, local)
    ) for im in im_list]
    done, pending = await asyncio.wait(tasks, return_when=asyncio.ALL_COMPLETED)
    return done


async def download_muti_im_(im_list: list, local="image"):
    print("多下载")
    tasks = [asyncio.create_task(
        download_with_im_(im, local)
    ) for im in im_list]
    done, pending = await asyncio.wait(tasks, return_when=asyncio.ALL_COMPLETED)
    return done

import json
class Cookie_image_getter:
    def __init__(self, tag="クッキー☆"):
        self.tag = tag
        self.random_cookie_list = []
        self.completed = []
        self.sended = []

    async def initial(self):
        self.check_usable()
        print("初始化")
        self.load_sended()
        while True:
            try:
                # self.random_cookie_list = get_cookie_elements(self.tag)
                await self.get_cookie_elements_to_num()
                print(self.random_cookie_list)
                await self.pick_some_cookies_to_download(4)
                break
            except Exception:
                if not self.random_cookie_list:
                    print("tag无效")
                    raise Exception
                print("网络错误，3s后进行重试")
                import time
                await asyncio.sleep(3)

        print("初始化完毕")

    def save_to_json(self):
        print("保存json", self.sended)
        with open("json_sended.json", 'w') as f:
            json.dump(self.sended, f)

    def load_sended(self):
        if os.path.exists("json_sended.json"):
            print("加载json")
            with open("json_sended.json", 'r') as f:
                self.sended += json.load(f)

    def get_out_of_sended(self):
        for elem in self.random_cookie_list:
            if elem in self.sended:
                self.random_cookie_list.remove(elem)

    async def get_cookie_elements_to_num(self, num=20):
        while len(self.random_cookie_list) < num:
            self.random_cookie_list += get_cookie_elements(self.tag)
            self.get_out_of_sended()

    def check_usable(self):
        if self.random_cookie_list is []:
            raise IndexError

    async def pick_some_cookies_to_download(self, some=5):
        self.get_out_of_sended()
        selected_elements = random.sample(self.random_cookie_list, some)
        self.random_cookie_list = [elem for elem in self.random_cookie_list if elem not in selected_elements]
        try:
            await download_muti_im(selected_elements)
        except Exception:
            await download_muti_im_(selected_elements)
        self.completed += selected_elements

    async def send_random_cookie(self, bot: Bot, event: Event):
        print("随机饼图，启动")
        if not self.completed:
            await bot.send(event, "初始化中，请稍后再试")
            return
        # print(self.completed)
        elem = random.sample(self.completed, 1)[0]
        print(elem, '发送')
        image_path = os.path.abspath('image')
        file_path = f'{image_path}/{elem}'
        if not is_full_color(file_path):
            try:
                print("重新发送")
                self.completed.remove(elem)
                os.remove(file_path)
            except PermissionError or ValueError:
                pass
            asyncio.create_task(self.pick_some_cookies_to_download(1))
            self.check_reload()
            await self.send_random_cookie(bot, event)
            return
        i = 0
        while i < 3:
            try:
                await send_image_from_ab_path(bot, event, file_path)
                i += 10
            except Exception:
                i += 1
        if i == 3:
            await bot.send(event, "发送失败，请重试")
        try:
            self.completed.remove(elem)
            os.remove(file_path)
        except PermissionError or ValueError:
            pass
        self.sended.append(elem)
        self.save_to_json()
        asyncio.create_task(self.pick_some_cookies_to_download(1))
        self.check_reload()

    def check_reload(self):
        while len(self.random_cookie_list) < 20:
            self.random_cookie_list += get_cookie_elements(self.tag)
            asyncio.create_task(self.pick_some_cookies_to_download(2))
            self.get_out_of_sended()


if __name__ == '__main__':
    b = Cookie_image_getter()
    asyncio.run(b.initial())
    pass



import asyncio
import time
from functools import wraps


def repeat_every_30_seconds(coro):
    @wraps(coro)
    async def wrapper(*args, **kwargs):
        try:
            while True:
                start_time = time.time()
                result = await coro(*args, **kwargs)
                elapsed_time = time.time() - start_time
                sleep_duration = max(30 - elapsed_time, 0)
                await asyncio.sleep(sleep_duration)
        except Exception as e:
            print(f"Error in {coro.__name__}: {e}")
            return
    return wrapper

def retry_async(retries=3, delay=1):
    def decorator(func):
        async def wrapper(*args, **kwargs):
            for _ in range(retries):
                try:
                    return await func(*args, **kwargs)
                except Exception as e:
                    print(f"Failed with {str(e)}, retrying...")
                    await asyncio.sleep(delay)
            return await func(*args, **kwargs)
        return wrapper
    return decorator


@repeat_every_30_seconds
async def main():
    print(1111)

if __name__ == '__main__':
    asyncio.run(main())
    print(222)



from datetime import datetime, timedelta
from nonebot.adapters.onebot.v11 import Event

def get_chat_id(event: Event):
    return str(event.get_user_id()) if event.is_tome() else str(event.group_id)

async def time_detect(input_time_str):
    input_time = datetime.strptime(input_time_str, "%Y年%m月%d日 %H：%M：%S")
    now = datetime.now()
    time_difference = now - input_time
    if time_difference < timedelta(minutes=1):
        return True
    else:
        return False


import json
from typing import Union
from nonebot.adapters.onebot.v11 import Bot, Event, MessageSegment, Message
import aiohttp
from lxml import etree
import os
import nonebot.log as logging

from nicoinfo.plugins.nicoinfo.nico_auth import subscriber
from nicoinfo.plugins.nicoinfo.utils import get_chat_id

logger = logging.logger


def video_info_to_str(last: dict):
    return f"""视频标题: {last['title']}
作者：{last['auth']}
视频日期: {last['date_info']}
sm号:{last['sm']}
视频链接: {last['url']}
视频简介： {last['description']}"""


async def download_image(url, local_path):
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            if response.status == 200:
                directory = os.path.dirname(local_path)
                if not os.path.exists(directory):
                    os.makedirs(directory)
                with open(local_path, 'wb') as file:
                    file.write(await response.read())


async def fetch(url):
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            if response.status == 200:
                html_content = await response.text()
                return html_content
            else:
                print(f"Error: {response.status}")
                return None


async def extract_og_image(url):
    html_content = await fetch(url)
    sm = url.split('/')[-1]
    if html_content:
        parser = etree.HTMLParser(encoding='utf-8')
        tree = etree.fromstring(html_content, parser)
        OGimage = tree.xpath('//meta[@property="og:image"]/@content')
        if OGimage:
            await download_image(str(OGimage[0]), f'./image/{sm}.png')
            return f'./image/{sm}.png'


def subscriptions_to_json_data(subscriptions: dict[str, subscriber]) -> dict:
    json_data_of_subscriptions = {}
    for chat_id in subscriptions:
        json_data_of_subscriptions[chat_id] = subscriptions[chat_id].get_json_data()
    return json_data_of_subscriptions


def json_data_to_subscriptions(json_data):
    subscriptions = {}
    for chat_id in json_data:
        subscriptions[chat_id] = subscriber.create_from_json_data(json_data[chat_id])
    return subscriptions


def load_subscriptions(filename):
    try:
        with open(filename, "r") as file:
            subscriptions = json.load(file)
    except FileNotFoundError:
        subscriptions = {}
    subscriptions = json_data_to_subscriptions(subscriptions)
    return subscriptions


def save_subscriptions(subscriptions, filename):
    subscriptions_data = subscriptions_to_json_data(subscriptions)
    with open(filename, "w") as file:
        json.dump(subscriptions_data, file)


# TODO：可能写错了
async def send_image_and_text(bot: Bot, event: Event, text: str, image: str):
    text = MessageSegment.text(text)
    local_image_path = image
    ab_local_image_path = os.path.abspath(local_image_path)
    local_image = MessageSegment.image(f"file:///{ab_local_image_path}")
    message = Message(text + local_image)
    await bot.send(event, message)


async def send_image_from_ab_path(bot: Bot, event: Event, path: str):
    image = MessageSegment.image(f"file:///{path}")
    await bot.send(event, image)


async def send_image_from_url(bot: Bot, event: Event, url: str):
    base_name = url.split('/')[-1]
    local_image_path = f"image/{base_name}"
    ab_local_image_path = os.path.abspath(local_image_path)
    await download_image(url, ab_local_image_path)
    local_image = MessageSegment.image(f"file:///{ab_local_image_path}")
    await bot.send(event, local_image)
    os.remove(ab_local_image_path)


async def get_samune_from_video(video: Union[dict, str]) -> str:
    if isinstance(video, dict):
        video = video['url']
    logger.debug("video信息：")
    logger.debug(video)
    return await extract_og_image(video)


# TODO:私人群聊，发图不发图，event
async def last_video_send_message(bot: Bot, send_image: bool, last_video: dict, event: Event | str | int,
                                  is_private: bool):
    if type(event) == Event:
        event = get_chat_id(event)
    text = video_info_to_str(last_video)
    if is_private:
        if not send_image:
            logger.info(text)
            await bot.send_private_msg(user_id=event, message=Message(text))
            return
        video = await get_samune_from_video(last_video)
        logger.debug("封面信息" + str(video))
        await send_image_and_text(bot, event, text, str(video))
        os.remove(video)


async def send_last_video_to_private_or_group(bot: Bot, event_id, last_video: dict, is_private: bool):
    text = video_info_to_str(last_video)
    local_image_path = await get_samune_from_video(last_video)
    ab_local_image_path = os.path.abspath(local_image_path)
    local_image = MessageSegment.image(f"file:///{ab_local_image_path}")
    message = Message(text + local_image)
    if is_private:
        await bot.send_private_msg(user_id=event_id, message=message)
    else:
        await bot.send_group_msg(group_id=event_id, message=message)


def video_info_to_str_BBsusume(last: dict):
    return f"""视频标题: {last['title']}
视频日期: {last['date_info']}  sm号:{last['sm']}
播放量：{last['view']}  评分：{last['point']}  弹幕数：{last['comment']}"""

async def send_last_video_to_private_or_group_BBsusume(bot: Bot, event_id, last_video: dict, is_private: bool):
    text = video_info_to_str_BBsusume(last_video)
    local_image_path = await get_samune_from_video(last_video)
    ab_local_image_path = os.path.abspath(local_image_path)
    local_image = MessageSegment.image(f"file:///{ab_local_image_path}")
    message = Message(text + local_image)
    if is_private:
        await bot.send_private_msg(user_id=event_id, message=message)
    else:
        await bot.send_group_msg(group_id=event_id, message=message)


async def send_last_video_to_private_or_group_fast(bot: Bot, event_id, last_video: dict, is_private: bool):
    text = video_info_to_str(last_video)
    if is_private:
        await bot.send_private_msg(user_id=event_id, message=text)
    else:
        await bot.send_group_msg(group_id=event_id, message=text)

from PIL import Image

def is_full_color(image_path):
    try:
        # 打开图像
        with Image.open(image_path) as img:
            # 如果图像不是RGB或RGBA格式，则它不是全彩色的
            if img.mode not in ['RGB', 'RGBA']:
                return False
            # 获取图像的三个通道
            r, g, b = img.split()
            # 检查每个通道的最大和最小值，以确定是否存在颜色变化
            for channel in [r, g, b]:
                if channel.getextrema() == (0, 0) or channel.getextrema() == (255, 255):
                    return False
            return True
    except Exception as e:
        print(f"Error processing image: {e}")
        return False



import asyncio
import os
import threading
from nonebot import get_driver, on_keyword
from nonebot.adapters.onebot.v11 import Bot, Event

from .BB_susume import BB_susume, susume_bot
from .config import Config
from .get_info import subscriptions
from .nico_auth import author_container, subscriber, subscribers_run, start_asyncio_loop
from . import get_info
from .random_cookie_image import Cookie_image_getter
from .usage import load_subscriptions
from .utils import get_chat_id


from nonebot.plugin import PluginMetadata

from .config import Config

__plugin_meta__ = PluginMetadata(
    name="n站信息发送-cookie",
    description="发送n站剧场和静画信息，默认发送cookie tag下的内容",
    usage="""指令列表（需要管理员权限）
<>为必填参数（可填多个）--为选填参数（不必写--）
last <user_uid> --fast	获取作者的最新投稿，选--fast不发送封面
sub <user_uid>	订阅作者
desub <user_uid>	取消订阅
list	列出当前订阅
关键字列表（任何人都可以使用）
随机剧场  随机发送剧场信息
随机饼图	随机发送一张饼图
订阅tag <n站tag> <生成的对应关键字>		订阅n站tag，订阅后发送关键字即可随机发送图片
删除<n站tag>	删除订阅的tag，需要先订阅才能起效""",
    config=Config,
)


global_config = get_driver().config
config = Config.parse_obj(global_config)


async def run_subscribe_update():
    await subscribers_run(get_info.subscriptions)


def start_asyncio_loop_init(loop):
    asyncio.set_event_loop(loop)
    loop.run_forever()


new_loop = asyncio.new_event_loop()
threading.Thread(target=start_asyncio_loop_init, args=(new_loop,)).start()


def init_cookie_image_getter():
    global a
    a = Cookie_image_getter()
    new_loop.call_soon_threadsafe(asyncio.create_task, a.initial())


init_thread = threading.Thread(target=init_cookie_image_getter)
init_thread.start()

video_getter_json_name = "video_getter.json"
if os.path.exists(video_getter_json_name):
    video_getter_data = load_subscriptions(video_getter_json_name)
    video_getter = susume_bot.json_to_class(video_getter_data)
else:
    video_getter = susume_bot()

random_cookie_video = on_keyword({"随机剧场", "random video"}, block=True, priority=9)


@random_cookie_video.handle()
async def r_c_v(bot: Bot, event: Event):
    global video_getter
    asyncio.create_task(video_getter.send_random_video_to_bot(bot, event))


random_cookie_send = on_keyword({"随机饼图", "random cookie"}, block=True, priority=10)


@random_cookie_send.handle()
async def r_c_s(bot: Bot, event: Event):
    if not init_thread.is_alive():
        try:
            asyncio.create_task(a.send_random_cookie(bot, event))
        except PermissionError or ValueError:
            await bot.send(event, "操作频繁，请稍后重试")
    else:
        await bot.send(event, "初始化未完成，请等待片刻")


start_update = on_keyword({"启动订阅更新", "启动订阅"}, block=True)


@start_update.handle()
async def update_start(bot: Bot, event: Event):
    chat_id = get_chat_id(event)
    if chat_id in subscriptions:
        subscriptions[chat_id].get_the_bot(bot)
        await bot.send(event, "订阅更新已开启")
    else:
        await bot.send(event, "还没有订阅任何作者")


stop_update = on_keyword({"停止订阅更新"}, block=True)


@stop_update.handle()
async def update_stop(bot: Bot, event: Event):
    chat_id = get_chat_id(event)
    if chat_id in subscriptions:
        subscriptions[chat_id].out_of_bot()
        await bot.send(event, "订阅更新已关闭")
    else:
        await bot.send(event, "还没有订阅任何作者")


sub_cookie_tag = on_keyword({"订阅tag"}, block=False, priority=6)


@sub_cookie_tag.handle()
async def sub_tag(bot: Bot, event: Event):
    args = str(event.get_message()).split(' ')
    try:
        tag = args[1]
        keyword_ = args[2]
        print(tag, keyword_)
    except IndexError:
        await bot.send(event, "参数错误")
        return
    try:
        b = Cookie_image_getter(tag=tag)
        asyncio.create_task(b.initial())
    except Exception:
        await bot.send(event, "tag无效")
        return

    async def send_cookie_twice(bot, event):
        try:
            await b.send_random_cookie(bot, event)
        except Exception:
            try:
                await b.send_random_cookie(bot, event)
            except Exception:
                await bot.send(event, "请稍后再试")

    chat_id = get_chat_id(event)
    b_activate = on_keyword({keyword_}, block=True, priority=7)

    @b_activate.handle()
    async def b_func(key_not: Bot, key_event: Event):
        chat_id_ = get_chat_id(key_event)
        if key_not == bot and chat_id_ == chat_id:
            print(keyword_, "启动")
            asyncio.create_task(send_cookie_twice(bot, event))

    b_del = on_keyword({"删除" + keyword_, "删除" + tag}, block=True, priority=2)

    @b_del.handle()
    async def def_b():
        print("")
        del b
        await bot.send(event, "已删除订阅")

    await asyncio.sleep(2)

    if not b.random_cookie_list:
        await bot.send(event, "tag无效")
        return

    await bot.send(event, "订阅成功")


use_inof = """指令列表（需要管理员权限）
<>为必填参数（可填多个）--为选填参数（不必写--）
last <user_uid> --fast	获取作者的最新投稿，选--fast不发送封面
sub <user_uid>	订阅作者
desub <user_uid>	取消订阅
list	列出当前订阅
关键字列表（任何人都可以使用）
随机饼图	随机发送一张饼图
订阅tag <n站tag> <生成的对应关键字>		订阅n站tag，订阅后发送关键字即可随机发送图片
删除<n站tag>	删除订阅的tag，需要先订阅才能起效"""

how_to_use = on_keyword({"bot用法", "机器人用法"}, block=False, priority=5)


@how_to_use.handle()
async def use_to_how(bot: Bot, event: Event):
    print("机器人用法")
    await bot.send(event, use_inof)



import asyncio
import random
from datetime import datetime
import requests
from lxml import etree
import math
from nonebot.adapters.onebot.v11 import Bot, Event

def get_chat_id(event: Event):
    return str(event.get_user_id()) if event.is_tome() else str(event.group_id)

try:
    from nicoinfo.plugins.nicoinfo.usage import get_samune_from_video
except ValueError:
    from usage import get_samune_from_video


def get_point_of_video(view, comment, like, mylist):
    return int(view) * 1 + int(comment) * 20 + int(like) * 20 + int(mylist) * 100


def time_difference_from_now(time_string: str) -> int:
    time_format = "%Y/%m/%d %H:%M"
    given_time = datetime.strptime(time_string, time_format)
    current_time = datetime.now()
    time_difference = current_time - given_time
    return int(time_difference.total_seconds())


def video_info_to_str(last: dict):
    return f"""视频标题: {last['title']}
视频日期: {last['date_info']}
sm号:{last['sm']}"""


class BB_susume:
    def __init__(self, tag="BBクッキー☆劇場", sort='h'):
        self.tag = tag
        self.link = "https://www.nicovideo.jp/tag/" + tag
        self.susumeta = []  # 会话：[已推荐列表]
        self.susume_list = {}
        self.sort = sort

    def init(self):
        while len(self.susume_list) < 20:
            self.get_random_video(5)

    def get_random_video(self, num=5):
        page = random.randint(1, 20)
        headers = {'User-Agent': 'Mozilla/5.0'}
        response = requests.get(self.link + f"?page={page}&sort={self.sort}&order=d", headers=headers)
        # print(self.link + f"?page={page}&sort={self.sort}&order=d")
        if response.status_code == 200:
            root = etree.HTML(response.content)
            sm_dic = {}
            elements = root.xpath('//li[@data-video-item]')
            for element in elements:
                if 'data-id' in element.attrib:
                    sm = element.attrib['data-id']
                elif 'data-video-id':
                    sm = element.attrib['data-video-id']
                    date = element.xpath('.//span[@class="time"]/text()')[0]
                    title = element.xpath('.//a[@title]/text()')[0]
                    view = element.xpath('.//li[@class="count view"]/span/text()')[0].replace(',', '')
                    comment = element.xpath('.//li[@class="count comment"]/span/text()')[0].replace(',', '')
                    like = element.xpath('.//li[@class="count like"]/span/text()')[0].replace(',', '')
                    mylist = element.xpath('.//li[@class="count mylist"]/span/text()')[0].replace(',', '')
                    point = get_point_of_video(view, comment, like, mylist)
                    log_time = math.log(time_difference_from_now(date), 36000000)
                    url = f"https://www.nicovideo.jp/watch/{sm}"
                    sm_dic[sm] = {'point': int(point / log_time), 'title': title, 'date_info': date, 'sm': sm, 'url': url, 'view': view, 'comment': comment}
                    # print(title, int(point / log_time))
            sorted_sm = sorted(sm_dic.keys(), key=lambda key: sm_dic[key]['point'], reverse=True)
            for i in range(num):
                self.susume_list[sorted_sm[i]] = sm_dic[sorted_sm[i]]
            # print(self.susume_list)


class susume_bot:
    def __init__(self, tag="BBクッキー☆劇場", member=0):
        self.tag = tag
        self.susume_func = BB_susume(self.tag)
        self.sub_list = {}
        self.member = member
        """
        sub_list: 
            member:{
                is_private
                is_open
                susume_list
                susumeta
            }
        """
        self.bot = None

    def get_the_bot(self, bot):
        self.bot = bot

    def out_of_bot(self):
        self.bot = None

    async def get_random_video_info(self):
        self.susume_func.init()
        random_sm = random.choice(list(self.susume_func.susume_list.keys()))
        random_video = self.susume_func.susume_list[random_sm]
        des = video_info_to_str(random_video)
        del self.susume_func.susume_list[random_sm]
        self.susume_func.susumeta.append(random_sm)
        return des, random_video

    async def send_random_video_to_bot(self, bot: Bot, event: Event):
        des, last = await self.get_random_video_info()
        chat_id = get_chat_id(event)
        if chat_id not in self.sub_list:
            self.sub_list[chat_id] = {'is_private': event.is_tome(), 'susume_list': self.susume_func.susume_list, 'susumeta': self.susume_func.susumeta}
        from nicoinfo.plugins.nicoinfo.usage import send_last_video_to_private_or_group_BBsusume as send_message
        await send_message(bot, chat_id, last, event.is_tome())


    def get_json_data(self):
        subscriber_json = {
            'tag': self.tag,
            'sub_list': self.sub_list
        }
        return subscriber_json

    @staticmethod
    def json_to_class(json: dict):
        subscriber_instance = susume_bot(json['tag'])
        subscriber_instance.sub_list = json['sub_list']
        return subscriber_instance


if __name__ == "__main__":
    a = susume_bot()
    asyncio.run(a.get_random_video_info())
